// Token.h   (c) adolfo@di-mare.com

/*  resultado
    Ejemplo de una analizador sintctico yylex(), implementado a mano 
    para la gramtica de la calculadora. En la prctica es necesario 
    reprogramar yylex() para que reconozca un conjunto de tokens y 
    lexemas diferente, pero la estructura Token+Extractor+yylex() puede 
    ustilizarse en ese contexto diferente.
*/

#ifndef Token_h
#define Token_h

#include <iostream.h>
#include <iomanip.h>
#include <cctype>

#include "Astring.h"
#if defined(__BORLANDC__)  // Compilando con Borland C++
    #include <bool.h>      // Define bool para BC++ v3.1 o inferior
#endif



class Token {
/*  Permite usar la pareja <Token, Lexema> que retorna el analizador
    lxico de un compilador [usualmente yylex()].
*/
public:
    enum Tipo {
        NUM = 321,
		PADRE_VACIO,
		OP_SUM,
		OP_MUL,		// podra ser otro, mientras no "choque"
        P_ABRE,		// con otros valores para tokens
        P_CIERRA,
		ID,
		ERROR  = 666, FIN  = 999
    };
    const Astring&  lexema() const { return _lexema; }
    const char*     Nombre() const { return Nombre(_tipo); }
    static const char* Nombre(int);


private:
    Tipo    _tipo; // Token::Tipo
    Astring _lexema;

	friend class Lista;

public:
	// Propiedades de asignacin
	void  prpTipo(Tipo s) { _tipo = s; }
	void  prpLexema(Astring s) { _lexema = s; }
	
    Token& operator  = (const Token&);  // copia

	friend bool operator <   (const Token&, const Token&);
	friend bool operator >   (const Token&, const Token&);
	friend bool operator ==  (const Token&, const Token&);
	
	friend ostream& operator << (ostream &salida, Token &B);
}; // Token

inline const char* Token::Nombre(int tipo) {
//  Retorna el nombre del token
    if      (tipo == Token::OP_SUM)   return "OP_SUM";
    else if (tipo == Token::OP_MUL)   return "OP_MUL";
    else if (tipo == Token::NUM)      return "NUM";
    else if (tipo == Token::P_ABRE)   return "P_ABRE";
    else if (tipo == Token::P_CIERRA) return "P_CIERRA";
    else if (tipo == Token::FIN)      return "FIN";
	else if (tipo == Token::PADRE_VACIO)      return "PADRE_VACIO";
    else                              return "ERROR";
    // no pude lograr usar un switch() con BC++ v3.1
} // Token::Nombre()

inline Token& Token::operator=(const Token& o) {
// Copia

    _tipo = o._tipo;
	_lexema = o._lexema;
	
    return *this;
}  // operator =

inline bool operator == (const Token &x, const Token &y) {
    return (x._tipo == y._tipo);
}  // operator ==

inline bool operator < (const Token &x, const Token &y) {
    return (x._tipo < y._tipo);
}  // operator <

inline bool operator > (const Token &x, const Token &y) {
    return (x._tipo > y._tipo);
}  // operator >


#endif // Token_h

// EOF: Token.h
